@@ -41,16 +41,16 @@ impl ProxyHttpStream {
 
         match cx.connect(domain, stream).await {
             Ok(s) => {
-                let negociated_h2 = match s.get_ref().negotiated_alpn() {
+                let negotiated_h2 = match s.get_ref().negotiated_alpn() {
                     Ok(Some(alpn)) => alpn == b"h2",
                     Ok(None) => false,
                     Err(err) => {
-                        let ierr = io::Error::new(ErrorKind::Other, format!("tls alpn negociate: {}", err));
+                        let ierr = io::Error::new(ErrorKind::Other, format!("tls alpn negotiate: {}", err));
                         return Err(ierr);
                     }
                 };
 
-                Ok(ProxyHttpStream::Https(s, negociated_h2))
+                Ok(ProxyHttpStream::Https(s, negotiated_h2))
             }
             Err(err) => {
                 let ierr = io::Error::new(ErrorKind::Other, format!("tls connect: {}", err));
@@ -106,7 +106,7 @@ impl ProxyHttpStream {
                 })
                 .with_no_client_auth();
 
-            // Try to negociate HTTP/2
+            // Try to negotiate HTTP/2
             config.alpn_protocols = vec![b"h2".to_vec(), b"http/1.1".to_vec()];
             Arc::new(config)
         });
@@ -126,9 +126,9 @@ impl ProxyHttpStream {
         let tls_stream = connector.connect(host, stream).await?;
 
         let (_, session) = tls_stream.get_ref();
-        let negociated_http2 = matches!(session.alpn_protocol(), Some(b"h2"));
+        let negotiated_http2 = matches!(session.alpn_protocol(), Some(b"h2"));
 
-        Ok(ProxyHttpStream::Https(tls_stream, negociated_http2))
+        Ok(ProxyHttpStream::Https(tls_stream, negotiated_http2))
     }
 
     #[cfg(not(any(feature = "local-http-native-tls", feature = "local-http-rustls")))]
@@ -140,7 +140,7 @@ impl ProxyHttpStream {
         Err(err)
     }
 
-    pub fn negociated_http2(&self) -> bool {
+    pub fn negotiated_http2(&self) -> bool {
         match *self {
             ProxyHttpStream::Http(..) => false,
             #[cfg(any(feature = "local-http-native-tls", feature = "local-http-rustls"))]
@@ -182,7 +182,7 @@ impl AsyncWrite for ProxyHttpStream {
 impl Connection for ProxyHttpStream {
     fn connected(&self) -> Connected {
         let conn = Connected::new();
-        if self.negociated_http2() {
+        if self.negotiated_http2() {
             conn.negotiated_h2()
         } else {
             conn
